"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Socket5Server = void 0;
const net_1 = require("net");
const utils_1 = require("../utils");
const createEmptyOptions = function () {
    return {
        host: '0.0.0.0',
        port: 1080
    };
};
class Socket5Server {
    /**
     * Create Socket 5
     * @example
     * new Socket5Server({})
     * @param opt
     */
    constructor(opt) {
        this.socket = (0, net_1.createServer)(this._handler.bind(this));
        this.option = Object.assign(createEmptyOptions(), opt);
        this.callback = (client) => {
            client.next();
        };
    }
    _handler(socket) {
        socket.once('data', data => this._authHandler(data, socket));
    }
    _authHandler(data, socket) {
        const version = data[0]; // Socket 版本
        if (version === 5) {
            if (this.option.auth) {
                socket.write(Buffer.from([version, 2])); // user auth
                socket.once('data', data => this._passwordAuth(data, socket));
            }
            else {
                socket.write(Buffer.from([version, 0])); // no auth
                socket.once('data', (chunk) => this.requestHandler(chunk, socket));
            }
        }
        else {
            socket.end();
            socket.destroy();
        }
    }
    _passwordAuth(data, socket) {
        var _a;
        const uLength = data[1];
        const username = data.subarray(2, 2 + uLength).toString('utf-8');
        const password = data.subarray(3 + uLength).toString('utf-8');
        if (username === ((_a = this.option.auth) === null || _a === void 0 ? void 0 : _a.user) && password === this.option.auth.pass) {
            socket.write(Buffer.from([5, 0])); // 认证通过
            socket.once('data', arg => this.requestHandler(arg, socket));
        }
        else {
            socket.write(Buffer.from([5, 1])); // 账号密码错误
            console.log('[Socket 5]: User: %s, Pass: %s Error.', username, password);
        }
    }
    /**
       * 处理客户端的请求
       * @param data
       * @param socket
       * @private
       */
    requestHandler(data, socket) {
        const version = data[0];
        const cmd = data[1]; // 0x01 CONNECT 连接
        if (cmd !== 1) {
            console.log('[Socket 5 Service]: Unsupported connection： %d', cmd);
            socket.end();
            socket.destroy();
        }
        else {
            if (version === 5 && cmd < 4 && data[2] === 0) {
                const type = data[3];
                const port = data.readUInt16BE(data.length - 2);
                const copyBuf = Buffer.from(data);
                if (type === 1) {
                    const host = this.hostname(data.subarray(4, 8));
                    this._end(host, port, copyBuf, socket);
                }
                else if (type === 3) {
                    const len = data[4];
                    const host = data.subarray(5, 5 + len).toString('utf8');
                    if ((0, utils_1.domainVerify)(host)) { // 验证域名
                        this._end(host, port, copyBuf, socket);
                    }
                    else {
                        socket.end();
                        socket.destroy();
                    }
                }
            }
            else { // 不知道的请求方式直接抛弃
                socket.end();
                socket.destroy();
            }
        }
    }
    _end(host, port, data, socket) {
        const nextFunc = () => {
            data[1] = 0x00;
            socket.write(data);
        };
        const info = {
            host,
            port,
            socket,
            next: nextFunc
        };
        this.callback(info);
    }
    hostname(buf) {
        if (buf.length === 4) {
            return buf.join('.');
        }
        else if (buf.length === 16) {
            return Array.from({ length: 8 }, (_, i) => buf.readUInt16BE(i * 2).toString(16)).join(':');
        }
        return '';
    }
    ;
    /**
       * start server
       * @param callback
       */
    listen(callback) {
        this.callback = callback;
        this.socket.listen(this.option.port, this.option.host, () => console.log(`[Socket 5]: Listening on ${this.option.port}`));
    }
}
exports.Socket5Server = Socket5Server;
